#!/usr/bin/env python
# -*- coding: utf-8 -*-
import websocket
import os
import hashlib
import json
import os
import netifaces
import psutil
import logging
import RepeatedTimer
import time
import shutil
import urllib.request
from cpuinfo import get_cpu_info

from functions import *
from config import *

#server = "wss://neromanagement.hypsoft.mx:443/ws/monitorEngx"
#report_interval = 5
#directories = ["/var/lib/engxMonitor"]
#files = ['/var/lib/engxMonitor/client.py','/var/lib/engxMonitor/config.py','/var/lib/engxMonitor/functions.py']
ws = None
rt = None

logging.basicConfig(format='%(asctime)s %(levelname)-8s %(message)s',
    level=logging.INFO,
    datefmt='%Y-%m-%d %H:%M:%S')

def on_message(ws, message):
#    logging.info(f"Received: {message}")
    mensajep = message.split(" ")
    command = mensajep[0]
    args = message[len(mensajep[0])+1:]
    if command == "doInit":
        doInit()
    elif command == "ackReport":
        if(is_json(args)):
            logging.info("Processing AckReport")
            argsj = json.loads(args)
            logging.info(f"args={argsj}")
            if 'success' in argsj and argsj['success']:
                if 'task' in argsj and argsj['task']:
                    if 'updateFiles' in argsj:
                        doRestart = False
                        updateFiles = argsj['updateFiles']
#                        logging.info(updateFiles)
                        if 'rfiles' in updateFiles:
                            doRestart = True
                            for pfile in updateFiles['rfiles']:
                                if os.path.exists(pfile):
                                    os.remove(pfile)
                                logging.info(f"Erasing file {pfile}")
                        if 'rdirs' in updateFiles:
                            doRestart = True
                            for pdir in updateFiles['rdirs']:
                                shutil.rmtree(pdir)
                                logging.info(f"Erasing directory {pdir}")
                        if 'dirs' in updateFiles:
                            for pdir in updateFiles['dirs']:
#                                logging.info(f"Checking directory {pdir}")
                                checkDirectory(pdir)
                        if 'update' in updateFiles:
                            doRestart = True
                            updateCRC = argsj['updateFiles']['updateC']
#                            logging.info(updateCRC)
                            for pfile in updateFiles['update']:
#                                logging.info(f"Updating file {pfile} from {argsj['updateURL']}{pfile}")
                                try:
                                    rfile = urllib.request.urlopen(f"{argsj['updateURL']}{pfile}")
                                except urllib.error.HTTPError as e:
                                    logging.info(f"HTTPError: {e.code} {argsj['updateURL']}{pfile}")
                                    doRestart = False
                                except urllib.error.URLError as e:
                                    logging.info(f"URLError: {e.reason} {argsj['updateURL']}{pfile}")
                                    doRestart = False
                                else:
                                    nfile = rfile.read()
                                    hfile = hashlib.md5(nfile).hexdigest()
                                    if updateCRC[updateFiles['update'].index(pfile)] == hfile:
                                        logging.info(f"Writing file {pfile}")
                                        with open(pfile, "w") as text_file:
                                            text_file.write(nfile.decode('utf-8'))
                                    else:
                                        logging.info(f"Error downloading file {pfile} from {argsj['updateURL']}{pfile}")
                        if 'updateDynamic' in updateFiles:
                            doRestart = True
                            for pfile in updateFiles['updateDynamic']:
                                logging.info(f"Updating file {pfile} from {updateFiles['updateDynamic'][pfile]}")
                                with open(pfile, "w") as text_file:
                                    text_file.write(updateFiles['updateDynamic'][pfile])
                        if doRestart:
                            logging.info("Restarting to apply changes")
                            logging.info("================================================")
                            quit()
                else:
                    logging.info(f"Task unsuccessfull in {command}")
            else:
                logging.info(f"No success in {command}")
        else:
            logging.info("AckResponse not in format")
    else:
        logging.info(f"Received unknown command: {command}")

def on_error(ws, error):
    rt.stop()
    if error != None:
        logging.info("ws error={}".format(error))
    else:
        logging.info("Normal exit")
    logging.info("Sleeping and trying to reconnect")
    time.sleep(2)
    logging.info("*************************************")
    quit()

def on_close(ws, close_status_code, close_msg):
    logging.info("Socket Closed")
    logging.info("Sleeping and trying to reconnect")
    logging.info("*************************************")
    logging(rt)
    logging(rt.is_running)
    rt.stop()
    time.sleep(2)
    quit()
#    connectWS()

def on_open(ws):
    logging.info("Socket Open")

def connectWS():
    global ws
    logging.info("*************************************")
    logging.info("Connecting a {}".format(server))
    ws = websocket.WebSocketApp(server,
                              on_open=on_open,
                              on_message=on_message,
                              on_error=on_error,
                              on_close=on_close)
    ws.run_forever()

def doInit():
    report()

def report():
    logging.info("Sending Report")
    report = {}
    report['command'] = 'report'
    report['status'] = {}
    report['status']['hostname'] = os.uname()[1]
    report['status']['inet'] = getINET()
    report['status']['cpu'] = getCPU()
    report['status']['mem'] = {'total' : psutil.virtual_memory().total, 'free' : psutil.virtual_memory().free, 'avail' : psutil.virtual_memory().available, 'used' : psutil.virtual_memory().used}
    report['status']['disk'] = getDisk()
    report['files'] = getFiles()
    report['localip'] = ws.sock.sock.getsockname()[0]
    #    logging.info(report['files'])
    #    logging.info(f"Sending {json.dumps(report)}")
    #    logging.info(ws)
    ws.send(json.dumps(report))
    if not rt.is_running:
        logging.info("Running start from report")
        rt.start()

rt = RepeatedTimer.RepeatedTimer(report_interval, report)

if __name__ == "__main__":
    connectWS()
