import os
import hashlib
import json
import os
import netifaces
import psutil
import logging
import apt
from cpuinfo import get_cpu_info

from config import *

def is_json(myjson):
  try:
    json.loads(myjson)
  except ValueError as e:
    return False
  return True

def checkDirectory(cdir):
    if not os.path.isdir(cdir):
        os.makedirs(cdir)

def getFiles():
    retval = {}
    for cdir in directories:
        checkDirectory(cdir)
    for cdir in directories:
        for path, subdirs, files in os.walk(cdir):
            for name in files:
                retval[os.path.join(path, name)] = getHash(os.path.join(path, name))
#                logging.info(f"f  {os.path.join(path, name)}")
#                logging.info(f"{os.path.join(path, name)}")
    return retval

def getINET():
    retval = {}
    for add in netifaces.interfaces():
        if add != "lo":
            addrs = netifaces.ifaddresses(add)
            macaddr = addrs[netifaces.AF_LINK]
            if netifaces.AF_INET in addrs:
                for ipv4 in addrs[netifaces.AF_INET]:
                    if 'ipv4' not in retval:
                        retval['ipv4'] = {}
                    if add not in retval['ipv4']:
                        retval['ipv4'][add] = {}
                    if 'addr' in ipv4:
                        retval['ipv4'][add]['addr'] = ipv4['addr']
                    if 'netmask' in ipv4:
                        retval['ipv4'][add]['netmask'] = ipv4['netmask']
                    if 'addr' in macaddr[0]:
                        retval['ipv4'][add]['mac'] = macaddr[0]['addr']
    return retval

def getDisk():
    retval = {}
    for mount in psutil.disk_partitions():
        if mount not in retval:
            mountu = psutil.disk_usage(mount.mountpoint)
            retval[mount.mountpoint] = {'device': mount.device, 'fs': mount.fstype}
            retval[mount.mountpoint]['usage'] = {'total': mountu.total, 'used': mountu.used, 'free':mountu.free}
    return retval

def getCPU():
    retval = {}
    load1, load5, load15 = [round(x / psutil.cpu_count() * 100,1) for x in psutil.getloadavg()]
    retval['load'] = {'1': load1, '5': load5, '15': load15}
    cpuinfo = get_cpu_info()
    if 'arch_string_raw' in cpuinfo:
        retval['arch'] = cpuinfo['arch_string_raw']
    if 'bits' in cpuinfo:
        retval['bits'] = cpuinfo['bits']
    if 'brand_raw' in cpuinfo:
        retval['brand_raw'] = cpuinfo['brand_raw']
    if 'stepping' in cpuinfo:
        retval['stepping'] = cpuinfo['stepping']
    if 'model' in cpuinfo:
        retval['model'] = cpuinfo['model']
    if 'family' in cpuinfo:
        retval['family'] = cpuinfo['family']
    return retval

def getHash(filename):
    retval = ''
    if os.path.isfile(filename):
        try:
            retval = hashlib.md5(open(filename,'rb').read()).hexdigest()
        except:
            retval = '00000000000000000000000000000000'
    else:
        retval = '00000000000000000000000000000000'
    return retval


def installPackage(pkg_name):
    retval = {}
    retval['success'] = False
    cache = apt.cache.Cache()
    cache.update()
    cache.open()
    pkg = cache[pkg_name]
    if pkg.is_installed:
        retval['success'] = True
    else:
        retval['success'] = True
        pkg.mark_install()
        try:
            cache.commit()
        except Exception as arg:
            retval['success'] = False
            retval['reason'] = f"{str(arg)}"
